#include <ArduinoJson.h>

/*
	* Copyright (c) 2021-11 by ELECTRONIC ASSEMBLY <technik@lcd-module.de>
	*
	* Weather implementation ESP32 and demo project EA uniTFTs
	* Weatherdata from https://openweathermap.org/
	*
	* This file is free software; you can redistribute it and/or modify
	* it under the terms of either the GNU General Public License version 2
	* or the GNU Lesser General Public License version 2.1, both as
	* published by the Free Software Foundation.
*/
#include "debug.h"
#include "weather.h"
#include "src/SmallProtocol/uniTFT.h"
#include "src/Unicode/Unicode.h"
#include "src/WiFi/wifi_ea.h"
#include <ETH.h>

//#include "src/HttpClient/HttpClient.h"
#include <HTTPClient.h>
#include <WiFiClient.h>
#include "ArduinoJson.h"
#include "uniTFTDesignerDefines.h"

const char *openWeatherMapAdress = "api.openweathermap.org";

const weathericon Weather::lookup[WEATHERELEMENTS] ={ //Fill look-up table for icons depending on weather situation
	weathericon{ 200, 221, "cloudyThunderyColor" },
	weathericon{ 230, 232, "sunnyThunderyColor" },
	weathericon{ 300, 300, "cloudyRainy1Color" },
	weathericon{ 301, 301, "cloudyRainy2Color" },
	weathericon{ 302, 302, "cloudyRainy3Color" },
	weathericon{ 310, 310, "cloudyRainy1Color" },
	weathericon{ 311, 311, "sunnyRainy1Color" },
	weathericon{ 312, 312, "sunnyRainy2Color" },
	weathericon{ 314, 314, "sunnyRainy3Color" },
	weathericon{ 321, 321, "cloudyRainy3Color" },
	weathericon{ 500, 500, "cloudyRainy1Color" },
	weathericon{ 501, 501, "cloudyRainy2Color" },
	weathericon{ 502, 504, "cloudyRainy3Color" },
	weathericon{ 511, 511, "cloudySnowy1Color" },
	weathericon{ 520, 520, "sunnyRainy1Color" },
	weathericon{ 521, 521, "sunnyRainy2Color" },
	weathericon{ 522, 531, "sunnyRainy3Color" },
	weathericon{ 600, 600, "cloudySnowy1Color" },
	weathericon{ 601, 601, "cloudySnowy2Color" },
	weathericon{ 602, 602, "cloudySnowy3Color" },
	weathericon{ 611, 611, "cloudySleetyColor" },
	weathericon{ 612, 615, "sunnySleetyColor" },
	weathericon{ 616, 616, "cloudySleetyColor" },
	weathericon{ 620, 620, "sunnySnowy1Color" },
	weathericon{ 621, 621, "sunnySnowy2Color" },
	weathericon{ 622, 622, "sunnySnowy3Color" },
	weathericon{ 701, 781, "fogColor" },
	weathericon{ 800, 800, "sunnyColor" },
	weathericon{ 801, 801, "sunnyCloudyColor" },
	weathericon{ 802, 802, "cloudyColor" },
	weathericon{ 803, 804, "verycloudyColor" }};
	
//----------------------------------------------------public Functions----------------------------------------------------
//Please use these functions in your sketch
/*----------------------------
	Func: decodeCommands
	Desc: Decode commands from uniTFTs concerning weather commands
	Vars: command
	Ret : ---
------------------------------*/
void Weather::decodeCommands(byte command)
{
	bool ok = false; //Returnvalue for weater and forecast
	switch(command)
	{
		case 'L': //Weather location
		{
			char type = uniTFT.getByte(); //Location type name or gps
			switch(type)
			{
				case WEATHER_LOCATION: //location as name
				{
					char16_t location[50];
					char lan[3];
					char appid[50];
					uniTFT.getString(location);
					uniTFT.getString(lan);
					char unit = uniTFT.getByte(); 
					uniTFT.getString(appid);
					ok = getStringForLocation(location,lan, unit, appid);
					if(ok) //only check forecast if weather data is available
						ok = getStringForLocationForecast(location,lan, unit, appid);
				}break;
				case WEATHER_GPSLOCATION: //location as gps
				{
					char latitude[10];
					char longitude[10];
					char lan[3];
					char appid[50];
					uniTFT.getString(latitude);
					uniTFT.getString(longitude);
					uniTFT.getString(lan);
					char unit = uniTFT.getByte(); 
					uniTFT.getString(appid);
					ok = getStringForGPSLocation(latitude,longitude,lan, unit, appid);
					if(ok) //only check forecast if weather data is available
						ok = getStringForGPSLocationForecast(latitude,longitude,lan, unit, appid);
				}break;
			}
			runMacro(sc_weather); //refresh screen content on uniTFTs
				
		}break;
		case 'N': //Refresh network list
		{
			#if (DBG_ON && DBG_WEATHER)
				Serial.println("Networklist"); 
			#endif
			getNetworkList(); //get a list of all WiFi's in the area
		}break;
		case 'W': //change Weather screen if WiFi is connected
		{
			if(WiFi_EA.isConnected()) //WiFi is connected
			{
				#if (DBG_ON && DBG_WEATHER)
					Serial.println("Show weather screen"); 
				#endif
				changeRegister(R_secCounter_Refresh, WEATHER_UPDATETIME_SEC);
				sprintf(szStr, "#MRN \"%s\";\x0a#MDS \"%s\";\x0a",sc_weather,GetCurrentWeather); //change to weather screen and call macro in uniTFT to get the actual weather depending on location
				boolean ret = uniTFT.sendDataPacket(szStr,strlen(szStr));
				#if (DBG_ON && DBG_WEATHER)
					Serial.printf("Packet: %s Ret: %d\r\n", szStr, ret); 
				#endif
			}
			else //WiFi unconnected
			{
				#if (DBG_ON && DBG_WEATHER)
					Serial.println("WiFi unconnected: Show select wifi screen"); 
				#endif
				sprintf(szStr, "#MRN \"%s\";\x0a",sc_WiFi); //Show screen for available WiFi nearby
				uniTFT.sendDataPacket(szStr,strlen(szStr));
			}
		}break;
		case 'S': //change signal strength indicator
		{
			char picName[20];
			int db = 0;
			if(WiFi_EA.isConnected() == false)
				sprintf(picName,"wifi0");
			else
			{
				db = WiFi_EA.RSSI(); //retrieve signal strength and select icon for it
				if(db == 1000)
					sprintf(picName,"ethernet");
				else
				{
					db = abs(db);
					if(db<50)
						sprintf(picName,"wifi4");
					else if(db<70)
						sprintf(picName,"wifi3");
					else if(db<90)
						sprintf(picName,"wifi2");
					else
						sprintf(picName,"wifi1");
				}
			}

			//update wifi signal strength indicator icon in header of screen sc_weather and sc_forecast
			sprintf(szStr, "#PPP %d, \"%s\"; 266, 229,3\x0A", pic_WiFi_strength, picName);
			uniTFT.sendDataPacket(szStr,strlen(szStr));
			#if (DBG_ON && DBG_WEATHER)
				Serial.printf("Change WiFi strength indicitor: %d\r\n", db);
				Serial.println(szStr);
			#endif
			
		}break;
		default:
		break;
	}
}

//----------------------------------------------------private Functions----------------------------------------------------

/*----------------------------
	Func: getNetworkList
	Desc: Get all WiFi networks and build new PictureBox
	Vars: ---
	Ret : ---
------------------------------*/
void Weather::getNetworkList() 
{
	sprintf(szStr, "#TCE0,%d,%d\x0a#ODI 15\x0a",sc_WiFi_bt_wifiSearch,sc_WiFi_bt_wifiNext); //Delete hour glass and deactivate button
	uniTFT.sendDataPacket(szStr,strlen(szStr));
	if(WiFi_EA.isConnected()) //WiFi is already connected, update actual status information on the right hand side of the screen
	{
		#if (DBG_ON && DBG_WEATHER)
			Serial.println("Updating WiFi Status"); 
			Serial.printf("Hostname: %s\r\n", WiFi_EA.Hostname());
		#endif
		sprintf(szStr, "#SSC%d,\"%s\"\x0A", sc_WiFi_lb_current_wifi, WiFi_EA.SSID());
		uniTFT.sendDataPacket(szStr,strlen(szStr));

		sprintf(szStr, "#SSC%d,\"%s\"\x0A", sc_WiFi_lb_current_name, WiFi_EA.Hostname());
		uniTFT.sendDataPacket(szStr,strlen(szStr));


		IPAddress ip = WiFi_EA.localIP(); 
		sprintf(szStr, "#SSC%d,\"IP: %03d.%03d.%03d.%03d\"\x0A", sc_WiFi_lb_current_ip, ip[0], ip[1], ip[2], ip[3]);
		uniTFT.sendDataPacket(szStr,strlen(szStr));

		sprintf(szStr, "#SSC%d,\"Port: %d\"\x0A", sc_WiFi_lb_current_port, SERVERPORT );
		uniTFT.sendDataPacket(szStr,strlen(szStr));
	}
	else //unconnected
	{
		sprintf(szStr, "#SSC%d,\"Not connected\"\x0A", sc_WiFi_lb_current_wifi);
		uniTFT.sendDataPacket(szStr,strlen(szStr));

		sprintf(szStr, "#ODI %d,%d, %d\x0A", sc_WiFi_lb_current_name, sc_WiFi_lb_current_ip, sc_WiFi_lb_current_port);
		uniTFT.sendDataPacket(szStr,strlen(szStr));
	}

	changeObjectVisibility(17,1); //Show box to select wifi's

	#if (DBG_ON && DBG_WEATHER)
		Serial.println("Start scan"); 
	#endif

	// scan for nearby networks:
	int numSsid = 0;
	numSsid = WiFi_EA.scanNetworks();
	#if (DBG_ON && DBG_WEATHER)
		Serial.printf("Found %d networks:\r\n", numSsid);
	#endif

	if (numSsid != -1) 
	{
		sprintf(szStr, "#EPS 15,771,6,4,27,3,17,2\x0a#EPD 15,20,20\x0a"); //Fill PictureBox
		uniTFT.sendDataPacket(szStr,strlen(szStr));

		for (int thisNet = 0; thisNet < numSsid; thisNet++) 
		{  
			char picName[20];
			int db = WiFi_EA.RSSI(thisNet); //retrieve signal strength and select icon for it
			if(db == 1000)
				sprintf(picName,"ethernet");
			else
			{
				db = abs(db);
				if(db<50)
					sprintf(picName,"wifi4");
				else if(db<70)
					sprintf(picName,"wifi3");
				else if(db<90)
					sprintf(picName,"wifi2");
				else
					sprintf(picName,"wifi1");
			}

			sprintf(szStr, "#EPA%d,\"%s\";\"%s\";\x0a",sc_WiFi_pb_wifiList,picName,WiFi_EA.SSID(thisNet)); //Add SSID together with signal icon in picturebox
			uniTFT.sendDataPacket(szStr,strlen(szStr));
			#if (DBG_ON && DBG_WEATHER)
				Serial.println(WiFi_EA.SSID(thisNet));
			#endif
		}


		sprintf(szStr, "#EPP15,15,195,1,4,5,5,8,5,10\x0a#MDT 15,;,<P:/macro/SelectWiFi.emc>,;\x0a#TCR0,1,15\x0a#EPO15,1\x0a",sc_WiFi_pb_wifiList); //Replace picturebox and connect touchmacro
		uniTFT.sendDataPacket(szStr,strlen(szStr));

	}

	changeObjectVisibility(17,0); //hide hour-glass

	sprintf(szStr, "#TCE1,%d,%d\x0a",sc_WiFi_bt_wifiSearch,sc_WiFi_bt_wifiNext); //enable touch button to search again
	uniTFT.sendDataPacket(szStr,strlen(szStr));
}

/*----------------------------
	Func: changeRegister
	Desc: Change register value (int)
	Vars: Register Id, value
	Ret : ---
------------------------------*/
void Weather::changeRegister(int id, int value)
{
	sprintf(szStr, "#VRI%d,%d\x0a",id,value);
	uniTFT.sendDataPacket(szStr,strlen(szStr));
}

/*----------------------------
	Func: changeRegister
	Desc: Change register value (float)
	Vars: Register Id, value
	Ret : ---
------------------------------*/
void Weather::changeRegister(int id, float value)
{
	sprintf(szStr, "#VRF%d,%f\x0a",id,value);
	uniTFT.sendDataPacket(szStr,strlen(szStr));
}

/*----------------------------
	Func: changeStringRegister
	Desc: Change StringRegister text 
	Vars: String Id, text
	Ret : ---
------------------------------*/
void Weather::changeStringRegister(int id, const char *text)
{
	sprintf(szStr, "#VSS%d,\"%s\";\x0a",id,text);
	uniTFT.sendDataPacket(szStr,strlen(szStr));
}

/*----------------------------
	Func: changeStringRegisterUnicode
	Desc: Change StringRegister text (unicode)
	Vars: String Id, text
	Ret : ---
------------------------------*/
void Weather::changeStringRegisterUnicode(int id, const char *text)
{
	sprintf(szStr, "#VSS%d,\"%s\";\x0a",id,text);
	int len = utf8_to_unicode((unsigned char*)szStr, sizeof(szStr), unicodeStr, 600);
	uniTFT.sendDataPacket((char*)unicodeStr,len*2);
}

/*----------------------------
	Func: changeArrayValue
	Desc: Change array value (int)
	Vars: array Id, index, value
	Ret : ---
------------------------------*/
void Weather::changeArrayValue(int id, int index, int value)
{
	sprintf(szStr, "#VAV%d,%d,%d\x0a",id,index,value);
	uniTFT.sendDataPacket(szStr,strlen(szStr));
}

/*----------------------------
	Func: changeArrayValue
	Desc: Change array value (float)
	Vars: Array Id, index, value
	Ret : ---
------------------------------*/
void Weather::changeArrayValue(int id, int index, float value)
{
	sprintf(szStr, "#VAV%d,%d,%f\x0a",id,index,value);
	uniTFT.sendDataPacket(szStr,strlen(szStr));
}

/*----------------------------
	Func: changeArrayValue
	Desc: Change array value (text)
	Vars: Array Id, index, text
	Ret : ---
------------------------------*/
void Weather::changeArrayValue(int id, int index, const char *text)
{
	sprintf(szStr, "#VAV%d,%d,\"%s\";\x0a",id,index,text);
	uniTFT.sendDataPacket(szStr,strlen(szStr));
}

/*----------------------------
	Func: changeObjectVisibility
	Desc: Change object visibility
	Vars: object Id, visibility
	Ret : ---
------------------------------*/
void Weather::changeObjectVisibility(int id, int visibility)
{
	sprintf(szStr, "#OVI%d,%d\x0a",visibility, id);
	uniTFT.sendDataPacket(szStr,strlen(szStr));
}

/*----------------------------
	Func: runMacro
	Desc: Run macro
	Vars: macroName
	Ret : ---
------------------------------*/
void Weather::runMacro(const char *macroName)
{
	sprintf(szStr, "#MRN\"%s\";\x0a",macroName);
	uniTFT.sendDataPacket(szStr,strlen(szStr));
}

/*----------------------------
	Func: updateWeather
	Desc: Weather data ready -> run macro "UpdateCurrentWeather"
	Vars: ---
	Ret : ---
------------------------------*/
void Weather::updateWeather()
{
	sprintf(szStr, "#MRN\"%s\";\x0a",UpdateCurrentWeather);
	uniTFT.sendDataPacket(szStr,strlen(szStr));
}

/*----------------------------
	Func: updateWeatherForecast
	Desc: Weather forecast data ready -> run macro "UpdateWeatherForecast"
	Vars: ---
	Ret : ---
------------------------------*/
void Weather::updateWeatherForecast()
{
	sprintf(szStr, "#MRN\"%s\";\x0a",UpdateWeatherForecast);
	uniTFT.sendDataPacket(szStr,strlen(szStr));
}
/*----------------------------
	Func: checkWeather
	Desc: Get current weather information based on global string buffer (call function "getStringForGPSLocation" or "getStringForLocation" previous)
	Vars: ---
	Ret : true on success
------------------------------*/
bool Weather::checkWeather()
{
	bool ok = false;
	
	WiFiClient  client; //Client needed to poll openweathermap
	if (!client.connect(openWeatherMapAdress, 80))  //
	{
		#if (DBG_ON && DBG_WEATHER)
			Serial.println("Weather server not available");
		#endif
		client.stop();
		return false;
	}
	
	#if (DBG_ON && DBG_WEATHER)
		Serial.println("Check weather... ");
		Serial.println(szStr);
	#endif	
	// This will send the request to the server szStr-Buffer filled in "getStringForGPSLocation()" or "getStringForLocation()"
	  client.print(String("GET ") + szStr + " HTTP/1.1\r\n" +
            "Host: " + openWeatherMapAdress + "\r\n" +
            "User-Agent: ESP32\r\n" +
            "Connection: close\r\n" + 
			"Pragma: no-cache\r\n" + 
			"Cache-Control: no-cache\r\n" + 
			"User-Agent: ESP32\r\n" + 
			"Accept: text/html,application/json\r\n\r\n");
	unsigned long milis_wait = millis() + 2000;
	while (client.available() == 0 && milis_wait >= millis())
		;
	
	String header = "", payload = "";
	bool recHeader = true;
	while(client.available()) 
	{
		String line = client.readStringUntil('\r');
		if (line.length() == 1 && payload.length() == 0) 
		{
			recHeader = false;
			continue;
		}
		if (recHeader)
		  header += line;
		else
		  payload += line;
	}
	client.stop();

	if(payload.length())   //Check if response is there
	{
		//String payload = http.getString(); //request return

		DynamicJsonDocument doc(2*payload.length());  //Return is a JSON document
		deserializeJson(doc, payload); //start reading the weather data

		JsonObject obj = doc.as<JsonObject>();

		JsonArray arr = obj["weather"].as<JsonArray>();
		JsonObject weather = arr[0];

		int weatherid = weather["id"];
		const char *weathermain = weather["main"];
		const char *weatherdescription = weather["description"];

		float lon = doc["coord"]["lon"];
		float lat = doc["coord"]["lat"];
		float temp = doc["main"]["temp"];
		int pressure = doc["main"]["pressure"];
		int humidity = doc["main"]["humidity"];
		float windspeed = doc["wind"]["speed"];
		int winddeg = doc["wind"]["deg"];
		const char* location = doc["name"];
		const char* test = doc["name"];  
		const char* code = doc["cod"];
		int clouds = doc["clouds"]["all"];
		float rain = doc["rain"]["1h"];
		//Weather data decoded
		changeRegister(R_ValidWeather,1); //Weather data is valid
		changeRegister(R_Temp,temp); //Set temperature in uniTFT
		changeRegister(R_Rain,rain); //Set rain in uniTFT
		changeRegister(R_Humidity,humidity); //Set humidity in uniTFT
		changeRegister(R_Pressure,pressure); //Set pressure in uniTFT
		changeRegister(R_WindSpeed,(float)(windspeed*3.6)); //Set wind in km/h in uniTFT
		changeRegister(R_WindDeg,winddeg); //wind direction in uniTFT
		changeStringRegisterUnicode(S_WearherDescription,weatherdescription); //Weather desctption in uniTFT
		changeStringRegisterUnicode(S_SearchLocation, location); //Set location name in uniTFT

		char pic[30];
		findWeatherPic(pic, weatherid); //get weather icon from look-up table
		#if (DBG_ON && DBG_WEATHER)
			Serial.printf("Wettericon: %s, ID: %d\r\n", pic, weatherid); 
		#endif	
		changeStringRegister(S_WeatherId,pic);  //change weather icon in uniTFT
		updateWeather(); //update screen in uniTFT showing actual weather
		ok = true;
	}
	else
	{
		changeRegister(R_ValidWeather,0); //Weather data is invalid
		ok = false;
		#if (DBG_ON && DBG_WEATHER)
			Serial.println("No internet connection"); 
		#endif	
	}
	//http.end();	//close http request
	
	return ok;
}

/*----------------------------
	Func: checkForecast
	Desc: Get current weather forecast information based on global string buffer (call function "getStringForGPSLocationForecast" or "getStringForLocationForecast" previous)
	Vars: ---
	Ret : true on success
------------------------------*/
bool Weather::checkForecast()
{
	bool ok = false;
	WiFiClient  client; //Client needed to poll openweathermap
	if (!client.connect(openWeatherMapAdress, 80))  //
	{
		#if (DBG_ON && DBG_WEATHER)
			Serial.println("Weather server not available");
		#endif
		client.stop();
		return false;
	}
	
	#if (DBG_ON && DBG_WEATHER)
		Serial.println("Check forecast... ");
		Serial.println(szStr);
	#endif	
	// This will send the request to the server szStr-Buffer filled in "getStringForGPSLocation()" or "getStringForLocation()"
	  client.print(String("GET ") + szStr + " HTTP/1.1\r\n" +
            "Host: " + openWeatherMapAdress + "\r\n" +
            "User-Agent: ESP32\r\n" +
            "Connection: close\r\n" + 
			"Pragma: no-cache\r\n" + 
			"Cache-Control: no-cache\r\n" + 
			"User-Agent: ESP32\r\n" + 
			"Accept: text/html,application/json\r\n\r\n");
	unsigned long milis_wait = millis() + 2000;
	while (client.available() == 0 && milis_wait >= millis())
		;
	
	String header = "", payload = "";
	bool recHeader = true;
	while(client.available()) 
	{
		String line = client.readStringUntil('\r');
		if (line.length() == 1 && payload.length() == 0) 
		{
			recHeader = false;
			continue;
		}
		if (recHeader)
		  header += line;
		else
		  payload += line;
	}
	client.stop();
	if(payload.length())   //Check if response is there
	{
		DynamicJsonDocument doc(2*payload.length());  //Return is a JSON document
		deserializeJson(doc, payload); //start reading the weather data

		JsonObject obj = doc.as<JsonObject>();
		JsonArray arr = obj["list"].as<JsonArray>();

		int tempMin = 9999; //needed for scaling the diagramm in uniTFT screen
		int tempMax = 0;
		int rainMax = 0;

		for(int i=0;i<10;i++) //Forecast has 10 individual weather information
		{
			JsonObject forecast = arr[i];


			JsonArray forecastArr = forecast["weather"].as<JsonArray>();
			JsonObject weather = forecastArr[0];

			int weatherid = weather["id"];
			const char *weathermain = weather["main"];
			const char *weatherdescription = weather["description"];

			float temp = forecast["main"]["temp"];
			float rain = forecast["rain"]["3h"];
			float probability = forecast["pop"];
			const char *forecastTime = forecast["dt_txt"];

			if(i==0) //forecast first value, set start time
			{
				char startTime[3] = { forecastTime[11], forecastTime[12] ,0};
				sprintf(szStr, "#VRI%d,%s\x0a",R_ForecastTime,startTime);
				uniTFT.sendDataPacket(szStr,strlen(szStr));
			}
			int min = (int)(temp-0.5);
			int max = (int)(temp+0.5);
			int rmax = (int)(rain+0.9);
			//Set parameters for scaling diagram
			if(tempMin>min) //Check if forecast temperature is lowest temperature
				tempMin = min;
			if(tempMax<max) //Check if forecast temperature is highest temperature
				tempMax = max; //do not use full scale, some distance needed
			if(rainMax<rmax) //Check if forecast rain is highest rainfall
				rainMax = rmax; //do not use full scale, some distance needed (more than temperature)
			#if (DBG_ON && DBG_WEATHER)
				Serial.printf("Temp: %0.1f TempMin %d TempMax: %d Rain %.1f\r\n", temp, tempMin, tempMax, rain);
			#endif	

			changeArrayValue(A_Temp,i, temp); //add temperature
			changeArrayValue(A_Rain,i,rain); //add rain
			changeArrayValue(A_Probability,i,(int)(probability*100)); //add probability to forecast array in uniTFT to be visualized in the diagram screen

			char pic[30];
			findWeatherPic(pic, weatherid); //get weather icon from look-up table
			changeArrayValue(A_WeatherId,i,pic); //add weathericon to forecast array in uniTFT. Icon is shown below diagramm
		}

		//Round temperature diagramm axis to the next 10s place
		tempMin-=5; 
		tempMax+=5; 
		tempMin = tempMin-(10-abs(tempMin%10)); 
		tempMax = tempMax+(abs(tempMax%10));
		float tempStep = (float)((tempMax-tempMin)/12.0);
		changeRegister(R_DiaTempMin,tempMin); //Change the Y-Axis minimum

		float rainStep = (float)(rainMax/6.0); //Calculate the Y-Axis rain step size
		changeRegister(R_DiaRainStep,(float)rainStep); //and send it

		if(tempStep<5) //Calculate the Y-Axis temperature step size
			changeRegister(R_DiaTempStep,(float)2.5);
		else
			changeRegister(R_DiaTempStep,(float)5.0);
		
		#if (DBG_ON && DBG_WEATHER)
			Serial.printf("Diagramm: Y-Temp %d->%d Step %.1f", tempMin, tempStep);
		#endif	

		changeRegister(R_ValidWeather,1); //Weather data is valid
		updateWeatherForecast(); //update forecast screen
		ok = true;
	}
	else
	{
		changeRegister(R_ValidWeather,0); //Weather data is invalid
		ok = false;
	}
	
	return ok;
}

/*----------------------------
	Func: findWeatherPic
	Desc: Retrieve picture name for weather
	Vars: buffer for name, ID for weather
	Ret : ---
------------------------------*/
void Weather::findWeatherPic(char* weatherPic, int weatherId)
{
	int i = 0;
	for(i; i<WEATHERELEMENTS;i++)
	{
		weathericon ico = lookup[i];	
		if(weatherId>=ico.id_start && weatherId<=ico.id_end)
		{
			sprintf(weatherPic, "%s",ico.picture);
			break;
		}
	}  
}

/*----------------------------
	Func: getStringForGPSLocation
	Desc: Build string for receiving weather informations (based on gps coordinates) from openweathermap and save it in global string buffer. 
	Vars: latitude, longitude, language, unit, appid
	Ret : true on success
------------------------------*/
bool Weather::getStringForGPSLocation(char *latitude, char *longitude, char *language, char unit, char *appid)
{
	char units[10];
	switch(unit)
	{
		case 1: sprintf(units,"metric");break;
		case 2: sprintf(units,"imperial");break;
		case 3: sprintf(units,"standard");break;
	}

	sprintf(szStr, "/data/2.5/weather?lat=%s&lon=%s&units=%s&lang=%s&appid=%s",latitude,longitude, units, language, appid);
	return checkWeather();
}

/*----------------------------
	Func: getStringForLocation
	Desc: Build string for receiving weather informations (based on location) from openweathermap and save it in global string buffer. 
	Vars: location, language, unit, appid
	Ret : true on success
------------------------------*/
bool Weather::getStringForLocation(char16_t *location, char *language, char unit, char *appid)
{
	char units[10];
	switch(unit)
	{
		case 1: sprintf(units,"metric");break;
		case 2: sprintf(units,"imperial");break;
		case 3: sprintf(units,"standard");break;
	}
	unsigned char utf8_location[50];

	unicode_to_utf8(location,sizeof(location)/ sizeof(location[0]),utf8_location,50);
	sprintf(szStr, "/data/2.5/weather?q=%s&units=%s&lang=%s&appid=%s",utf8_location, units, language, appid);
	return checkWeather();
}

/*----------------------------
	Func: getStringForGPSLocationForecast
	Desc: Build string for receiving weather forecast informations (based on gps coordinates) from openweathermap and save it in global string buffer. 
	Vars: latitude, langitude, language, unit, appid
	Ret : true on success
------------------------------*/
bool Weather::getStringForGPSLocationForecast(char *latitude, char *longitude, char *language, char unit, char *appid)
{
	char units[10];
	switch(unit)
	{
		case 1: sprintf(units,"metric");break;
		case 2: sprintf(units,"imperial");break;
		case 3: sprintf(units,"standard");break;
	}

	sprintf(szStr, "/data/2.5/forecast?lat=%s&lon=%s&units=%s&lang=%s&cnt=10&appid=%s",latitude,longitude, units, language, appid);
	return checkForecast();
}

/*----------------------------
	Func: getStringForLocationForecast
	Desc: Build string for receiving weather forecast informations (based on location) from openweathermap and save it in global string buffer. 
	Vars: location, language, unit, appid
	Ret : true on success
------------------------------*/
bool Weather::getStringForLocationForecast(char16_t *location, char *language, char unit, char *appid)
{
	char units[10];
	switch(unit)
	{
		case 1: sprintf(units,"metric");break;
		case 2: sprintf(units,"imperial");break;
		case 3: sprintf(units,"standard");break;
	}
	unsigned char utf8_location[50];

	unicode_to_utf8(location,sizeof(location)/ sizeof(location[0]),utf8_location,50);
	sprintf(szStr, "/data/2.5/forecast?q=%s&units=%s&lang=%s&cnt=10&appid=%s",utf8_location, units, language, appid);
	return checkForecast();
}
